@@ -19,7 +19,6 @@ pub(crate) fn merge(backdrop: Color8, src: Color8, opacity: u8) -> Color8 {
     let res_r;
     let res_g;
     let res_b;
-    let res_a;
 
     if back_a == 0 {
         res_r = src_r;
@@ -34,7 +33,7 @@ pub(crate) fn merge(backdrop: Color8, src: Color8, opacity: u8) -> Color8 {
         res_g = blend8(back_g, src_g, opacity);
         res_b = blend8(back_b, src_b, opacity);
     }
-    res_a = blend8(back_a, src_a, opacity);
+    let res_a = blend8(back_a, src_a, opacity);
     if res_a == 0 {
         Rgba([0, 0, 0, 0])
     } else {
@@ -684,13 +683,11 @@ fn test_set_saturation() {
                     // println!("set_saturation({:.3}, {:.3}, {:.3}, {:.3}) => ({:.3}, {:.3}, {:.3}) => sat: {:.5} (input sat: {:.5})",
                     // r, g, b, sat, r1, g1, b1, sat1, sat0);
 
-                    if !(r == g && g == b) {
-                        if (sat1 - sat).abs() > 0.00001 {
-                            panic!(
+                    if !(r == g && g == b) && (sat1 - sat).abs() > 0.00001 {
+                        panic!(
                                 "set_saturation({:.3}, {:.3}, {:.3}, {:.3}) => ({:.3}, {:.3}, {:.3}) => sat: {:.5} (input sat: {:.5})",
                                 r, g, b, sat, r1, g1, b1, sat1, sat0
                             );
-                        }
                     }
                 }
             }
@@ -801,7 +798,7 @@ fn test_normal() {
     assert_eq!(Rgba([118, 162, 135, 255]), res);
 }
 
-fn mul_un8(a: i32, b: i32) -> u8 {
+pub(crate) fn mul_un8(a: i32, b: i32) -> u8 {
     let t = a * b + 0x80;
     let r = ((t >> 8) + t) >> 8;
     r as u8
